
/* cmdline.c -- written by Istvan Varga, Apr 2002 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "midi_out.h"
#include "draw_win.h"
#include "events.h"
#include "cmdline.h"

/* print program usage */

void	print_usage (void)
{
	fprintf (stderr, "2D MIDI controller utility by Istvan Varga ");
	fprintf (stderr, "(Apr 2002)\n\n");
	fprintf (stderr, "Usage: controller [options ...]\n");
	fprintf (stderr, "The following options are available:\n");
	fprintf (stderr, " -h | -help | --help\n");
	fprintf (stderr, "    Print this message.\n");
	fprintf (stderr, " -o | --output <filename>\n");
	fprintf (stderr, "    Output file name (defaults to standard ");
	fprintf (stderr, "output.\n");
	fprintf (stderr, " -7 | --7-bit\n");
	fprintf (stderr, "    Send 7 bit control messages (this is the ");
	fprintf (stderr, "default).\n");
	fprintf (stderr, " -14 | --14-bit\n");
	fprintf (stderr, "    Send 14 bit control messages (LSB is sent ");
	fprintf (stderr, "in controller + 32).\n");
	fprintf (stderr, " -s | --size <2..8>\n");
	fprintf (stderr, "    Set window size to 128 times the value ");
	fprintf (stderr, "specified (default: 4).\n");
	fprintf (stderr, "Keyboard usage:\n");
	fprintf (stderr, "  F1 - F4\n");
	fprintf (stderr, "    Select controller for X output.\n");
	fprintf (stderr, "  F9 - F12\n");
	fprintf (stderr, "    Select controller for Y output.\n");
	fprintf (stderr, "  F5\n");
	fprintf (stderr, "    Print current settings.\n");
	fprintf (stderr, "  F7, F8\n");
	fprintf (stderr, "    Change MIDI channel.\n");
	fprintf (stderr, "  Left/Right/Up/Down arrows\n");
	fprintf (stderr, "    Set sleep time (minimum interval between ");
	fprintf (stderr, "updates).\n");
}

/* parse command line options */

void	parse_options (int argc, char **argv)
{
	int	i, j, k;

	j = argc; k = 1;
	while (--j) {
		if ((!(strcmp (argv[k], "-h")))		/* print usage */
		    || (!(strcmp (argv[k], "-help")))
		    || (!(strcmp (argv[k], "--help")))) {
			print_usage ();
			exit (0);
		/* window size */
		} else if ((!(strcmp (argv[k], "-s")))
			   || (!(strcmp (argv[k], "--size")))) {
			if (--j < 1) {
				fprintf (stderr, "missing argument for ");
				fprintf (stderr, "--size\n");
				print_usage ();
				exit (-1);
			}
			i = (int) atoi (argv[++k]);
			if ((i < 2) || (i > 8)) {
				fprintf (stderr, "invalid size\n");
				print_usage ();
				exit (-1);
			}
			cur_size = i;
		/* 14 bit ctrl */
		} else if ((!(strcmp (argv[k], "-14")))
			   || (!(strcmp (argv[k], "--14-bit")))) {
			out_format = 1;
		/* 7 bit ctrl */
		} else if ((!(strcmp (argv[k], "-7")))
			   || (!(strcmp (argv[k], "--7-bit")))) {
			out_format = 0;
		/* output file */
		} else if ((!(strcmp (argv[k], "-o")))
			   || (!(strcmp (argv[k], "--output")))) {
			if (--j < 1) {
				fprintf (stderr, "missing argument for ");
				fprintf (stderr, "--output\n");
				print_usage ();
				exit (-1);
			}
			outfile_name = argv[++k];
		} else {
			fprintf (stderr, "invalid option: %s\n", argv[k]);
			print_usage ();
			exit (-1);
		}
		k++;
	}
}

