
/* draw_win.c -- written by Istvan Varga, Apr 2002 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <gtk/gtk.h>
#include "draw_win.h"
#include "events.h"
#include "midi_out.h"

unsigned int	color0 = 0x00FFFFFF;	/* background			*/
unsigned int	color1 = 0x00C0C0C0;	/* normal line			*/
unsigned int	color2 = 0x00808080;	/* 8x line			*/
unsigned int	color3 = 0x00A0FFFF;	/* ctrl=64 area			*/

int		cur_size = 4;		/* current field size		*/

int		img_width;		/* width in pixels		*/
int		img_height;		/* height in pixels		*/

GdkVisual	*visual;
GdkGC		*gc1;
GdkWindow	*window;

GdkCursor	*cursor0, *cursor1;

GdkWindowAttr	attr;
char            win_title[16] = "MIDI controller";

/* calculate image width and height from current size */

void	get_size (void)
{
	img_height = cur_size * 128;
	img_width = cur_size * 128;
}

/* open window */

void	create_window (void)
{
	GdkColor	c;

	gdk_rgb_init ();
	/* calculate image size */
	get_size ();
	/* initialize window */
	visual = gdk_visual_get_best ();
	attr.title = win_title;
	attr.event_mask = GDK_ALL_EVENTS_MASK;
	attr.x = 0; attr.y = 0;
	attr.width = img_width; attr.height = img_height;
	attr.wclass = GDK_INPUT_OUTPUT;
	attr.visual = visual;
	attr.colormap = gdk_colormap_get_system ();
	attr.window_type = GDK_WINDOW_TOPLEVEL;
	attr.cursor = NULL;
	attr.wmclass_name = "controller";
	attr.wmclass_class = "main";
	attr.override_redirect = FALSE;
	/* create new window and make it visible */
	window = gdk_window_new (NULL, &attr, 0);
	c.red = (unsigned short) ((color0 & 0x00FF0000) >> 8);	/* set	   */
	c.green = (unsigned short) (color0 & 0x0000FF00);	/* backgnd */
	c.blue = (unsigned short) ((color0 & 0x000000FF) << 8);	/* color   */
	c.pixel = gdk_rgb_xpixel_from_rgb (color0);
	gdk_window_set_background (window, &c);
	gdk_window_show (window);	/* make window visible */
	gc1 = gdk_gc_new (window);
	gdk_gc_set_function (gc1, GDK_COPY);
	gdk_gc_set_fill (gc1, GDK_SOLID);
	gdk_rgb_gc_set_foreground (gc1, 0x00000000);
	gdk_rgb_gc_set_background (gc1, 0x00FFFFFF);
	/* cursors */
	cursor0 = gdk_cursor_new (GDK_TOP_LEFT_ARROW);
	cursor1 = gdk_cursor_new (GDK_CROSSHAIR);
	gdk_window_set_cursor (window, cursor1);
}

/* close window */

void	close_window (void)
{
	/* cursors */
	gdk_cursor_destroy (cursor0);
	gdk_cursor_destroy (cursor1);
}

/* draw window */

void	draw_window (void)
{
	int	i, l;

	/* ctrl=64 area */
	gdk_rgb_gc_set_foreground (gc1, color3);
	gdk_draw_rectangle (window, gc1, 1, 0, 63 * cur_size + 1,
					    img_width, cur_size - 1);
	gdk_draw_rectangle (window, gc1, 1, 64 * cur_size, 0,
					    cur_size - 1, img_height);
	/* normal lines */
	gdk_rgb_gc_set_foreground (gc1, color1);
	l = -1;
	i = 0;
	do {
		l += cur_size;
		if ((i & 7) == 7) continue;	/* skip 8x lines */
		gdk_draw_line (window, gc1, 0, img_height - l - 1,
					    img_width - 1, img_height - l - 1);
		gdk_draw_line (window, gc1, l, 0, l, img_height - 1);
	} while (++i < 128);
	/* 8x lines */
	gdk_rgb_gc_set_foreground (gc1, color2);
	l = -1;
	i = 0;
	do {
		l += (cur_size << 3);
		gdk_draw_line (window, gc1, 0, img_height - l - 1,
					    img_width - 1, img_height - l - 1);
		gdk_draw_line (window, gc1, l, 0, l, img_height - 1);
	} while ((i += 8) < 128);
}

/* get mouse pointer coordinates, and convert to controller values	*/
/* return -1, -1 if pointer is not in window area			*/

void	get_coord (int *xc, int *yc)
{
	gint		x, y;
	GdkModifierType	mask;

	mask = GDK_MODIFIER_MASK;
	gdk_window_get_pointer (window, &x, &y, &mask);
	/* check for valid coordinates */
	if ((x < 0) || (x >= img_width) || (y < 0) || (y >= img_height)) {
		*xc = *yc = -1;
		return;
	}
	y = img_height - y - 1;	/* flip vertically */
	if (out_format) {
		*xc = ((int) x << 7) / cur_size;	/* 14 bit mode */
		if (*xc > 16383) *xc = 16383;
		*yc = ((int) y << 7) / cur_size;
		if (*yc > 16383) *yc = 16383;
	} else {
		*xc = x / cur_size;			/* 7 bit mode */
		if (*xc > 127) *xc = 127;
		*yc = y / cur_size;
		if (*yc > 127) *yc = 127;
	}
}

