
/* events.c -- written by Istvan Varga, Apr 2002 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>
#include "draw_win.h"
#include "midi_out.h"
#include "events.h"

GdkEvent	*event;
int		sleep_ms = 10;		/* sleep time in ms		*/
int		button_pressed = 0;	/* mouse button state		*/

/* add n to controler X with range checking and print new value */

void	adjust_ctrl_X (int n)
{
	ctrl_X += n;
	if (ctrl_X < 0) ctrl_X = 0;
	if (out_format) {
		if (ctrl_X > 95) ctrl_X = 95;	/* limit in 14 bit mode */
		fprintf (stderr, "controller X: ");
		fprintf (stderr, "MSB: %d, LSB: %d\n", ctrl_X, ctrl_X + 32);
	} else {
		if (ctrl_X > 127) ctrl_X = 127;
		fprintf (stderr, "controller X: %d\n", ctrl_X);
	}
}

/* add n to controler Y with range checking and print new value */

void	adjust_ctrl_Y (int n)
{
	ctrl_Y += n;
	if (ctrl_Y < 0) ctrl_Y = 0;
	if (out_format) {
		if (ctrl_Y > 95) ctrl_Y = 95;	/* limit in 14 bit mode */
		fprintf (stderr, "controller Y: ");
		fprintf (stderr, "MSB: %d, LSB: %d\n", ctrl_Y, ctrl_Y + 32);
	} else {
		if (ctrl_Y > 127) ctrl_Y = 127;
		fprintf (stderr, "controller Y: %d\n", ctrl_Y);
	}
}

/* add n to channel with range checking and print new value */

void	adjust_channel (int n)
{
	set_channel (channel + n);
	fprintf (stderr, "channel: %d\n", channel);
}

/* add n to sleep time with range checking and print new value */

void	adjust_sleep_ms (int n)
{
	sleep_ms += n;
	if (sleep_ms < 1) sleep_ms = 1;
	if (sleep_ms > 100) sleep_ms = 100;
	fprintf (stderr, "sleep time: %d ms\n", sleep_ms);
}

/* function keys */

void	fkeys_func (int key)
{
	switch (key) {
	/* controller X */
	case GDK_F1:  adjust_ctrl_X (-8); return;
	case GDK_F2:  adjust_ctrl_X (-1); return;
	case GDK_F3:  adjust_ctrl_X ( 1); return;
	case GDK_F4:  adjust_ctrl_X ( 8); return;
	/* controller Y */
	case GDK_F9:  adjust_ctrl_Y (-8); return;
	case GDK_F10: adjust_ctrl_Y (-1); return;
	case GDK_F11: adjust_ctrl_Y ( 1); return;
	case GDK_F12: adjust_ctrl_Y ( 8); return;
	/* sleep time */
	case GDK_Down:	adjust_sleep_ms (-10); return;
	case GDK_Left:	adjust_sleep_ms ( -1); return;
	case GDK_Right:	adjust_sleep_ms (  1); return;
	case GDK_Up:	adjust_sleep_ms ( 10); return;
	/* channel */
	case GDK_F7:  adjust_channel (-1); return;
	case GDK_F8:  adjust_channel ( 1); return;
	/* print settings */
	case GDK_F5:				/* print current settings */
		fprintf (stderr, "==== current settings: ====\n");
		fprintf (stderr, "MIDI channel: %d\n", channel);
		if (out_format) {
			fprintf (stderr, "controller X: MSB: %d, LSB: %d\n",
					 ctrl_X, ctrl_X + 32);
			fprintf (stderr, "controller Y: MSB: %d, LSB: %d\n",
					 ctrl_Y, ctrl_Y + 32);
		} else {
			fprintf (stderr, "controller X: %d\n", ctrl_X);
			fprintf (stderr, "controller Y: %d\n", ctrl_Y);
		}
		fprintf (stderr, "sleep time: %d ms\n", sleep_ms);
	}
}

/* read and process keyboard and mouse events		*/
/* return status is the sum of the following values:	*/
/*	1: window should be redrawn			*/
/*	2: close window and exit			*/

int	get_events (void)
{
	int	rval;
	int	keynum, press, release, xc, yc;

	rval = 0;
	while ((event = gdk_event_get ()) != NULL) {
		if ((event->type == GDK_DELETE)		/* close window */
		    || (event->type == GDK_DESTROY)) {
			rval |= 2; continue;
		}
		if ((event->type == GDK_KEY_PRESS)	/* key press or    */
		    || (event->type == GDK_KEY_RELEASE)) {	/* release */
			keynum = event->key.keyval;
			/* key state */
			press = (event->type == GDK_KEY_PRESS ? 1 : 0);
			release = 1 - press;
			switch (keynum) {
			case GDK_F1:			/* function keys */
			case GDK_F2:
			case GDK_F3:
			case GDK_F4:
			case GDK_F5:
			case GDK_F7:
			case GDK_F8:
			case GDK_F9:
			case GDK_F10:
			case GDK_F11:
			case GDK_F12:
			case GDK_Left:
			case GDK_Right:
			case GDK_Up:
			case GDK_Down:
				if (press) fkeys_func (keynum);
			}
		} else if ((event->type == GDK_BUTTON_PRESS)	/* mouse */
			   || (event->type == GDK_2BUTTON_PRESS)
			   || (event->type == GDK_3BUTTON_PRESS)) {
			button_pressed = 1;
			gdk_window_set_cursor (window, cursor0);
		} else {				/* other events */
			if (event->type == GDK_BUTTON_RELEASE) {
				button_pressed = 0;
				gdk_window_set_cursor (window, cursor1);
			}
			/* check mouse pointer coordinates */
			get_coord (&xc, &yc);
			if (!button_pressed) {
				if ((xc >= 0) && (xc != ctrl_X_val)) {
					send_control_change (ctrl_X, xc);
					ctrl_X_val = xc;
				}
				if ((yc >= 0) && (yc != ctrl_Y_val)) {
					send_control_change (ctrl_Y, yc);
					ctrl_Y_val = yc;
				}
			}
			/* update window if necessary */
			if ((event->type == GDK_CONFIGURE)
			    || (event->type == GDK_EXPOSE)
			    || (event->type == GDK_MAP)) {
				rval |= 1;
			}
		}
	}

	return rval;
}

