
/* 2D MIDI controller for Linux / Gtk-1.2.x
 * 
 *     Copyright (c) 2002 Istvan Varga
 * 
 *     Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation files
 * (the "Software"), to deal in the Software without restriction,
 * including without limitation the rights to use, copy, modify, merge,
 * publish, distribute, sublicense, and/or sell copies of the Software,
 * and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 * 
 *     The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 *     THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 * 
 *     Except as contained in this notice, the name of the author
 * shall not be used in advertising or otherwise to promote the sale, use
 * or other dealings in this Software without prior written authorization
 * from the author.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <gdk/gdk.h>
#include <math.h>
#include "midi_out.h"
#include "draw_win.h"
#include "events.h"
#include "cmdline.h"

int	main (int argc, char **argv)
{
	int	end;

	/* default output file */
	outfile = stdout;
	/* disable stdout buffering */
	setvbuf (stdout, (char *) NULL, _IONBF, 0);
	/* parse command line options */
	parse_options (argc, argv);
	/* open output file if specified on command line */
	if (outfile_name != NULL) {
		outfile = fopen (outfile_name, "wb");
		if (outfile == NULL) {
			fprintf (stderr, "error opening output file\n");
			exit (-1);
		}
		setvbuf (outfile, (char *) NULL, _IONBF, 0);
	}
	/* initialize GDK and open window */
	gdk_init (&argc, &argv);
	create_window ();
	draw_window ();

	end = 0; do {
		end = get_events ();
		if (end & 1) draw_window ();
		usleep (1000 * sleep_ms);
	} while (!(end & 2));

	/* close window */
	close_window ();
	/* close output file if it is not stdout */
	if (outfile_name != NULL) {
		if (fclose (outfile)) {
			fprintf (stderr, "error closing output file\n");
			exit (-1);
		}
	}

	gdk_exit (0);
	return 0;
}

