
/* cmdline.c -- written by Istvan Varga, Apr 2002 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "midi_out.h"
#include "draw_keys.h"
#include "events.h"
#include "cmdline.h"

/* print program usage */

void	print_usage (void)
{
	fprintf (stderr, "Virtual MIDI keyboard utility by Istvan Varga ");
	fprintf (stderr, "(Apr 2002)\n\n");
	fprintf (stderr, "Usage: keyboard [options ...]\n");
	fprintf (stderr, "The following options are available:\n");
	fprintf (stderr, " -h | -help | --help\n");
	fprintf (stderr, "    Print this message.\n");
	fprintf (stderr, " -o | --output <filename>\n");
	fprintf (stderr, "    Output file name (defaults to standard ");
	fprintf (stderr, "output).\n");
	fprintf (stderr, " -M | --midi\n");
	fprintf (stderr, "    Write MIDI messages to output (this is the ");
	fprintf (stderr, "default).\n");
	fprintf (stderr, " -S | --score\n");
	fprintf (stderr, "    Write Csound score events to output.\n");
	fprintf (stderr, " -w | --winsize <0..3>\n");
	fprintf (stderr, "    Select window size:\n");
	fprintf (stderr, "      0: 450x45\n");
	fprintf (stderr, "      1: 600x60 (default)\n");
	fprintf (stderr, "      2: 750x75\n");
	fprintf (stderr, "      3: 1050x105\n");
	fprintf (stderr, "\n");
	fprintf (stderr, "Keyboard usage:\n");
	fprintf (stderr, "  1 - \\, Q - ], A - \', Z - /\n");
	fprintf (stderr, "    Notes to play; velocity can be selected by ");
	fprintf (stderr, "Shift, Ctrl, or Alt.\n");
	fprintf (stderr, "  Left/Right/Up/Down arrows\n");
	fprintf (stderr, "    Adjust note-on velocity (also selectable by ");
	fprintf (stderr, "Shift, Ctrl, and Alt).\n");
	fprintf (stderr, "  F1 - F4\n");
	fprintf (stderr, "    Set transpose value.\n");
	fprintf (stderr, "  Shift + F1 - F4\n");
	fprintf (stderr, "    Adjust note-off velocity.\n");
	fprintf (stderr, "  F9 - F12\n");
	fprintf (stderr, "    Select MIDI program.\n");
	fprintf (stderr, "  Shift + F9 - F12\n");
	fprintf (stderr, "    Select MIDI channel.\n");
	fprintf (stderr, "  F5\n");
	fprintf (stderr, "    Print current settings.\n");
	fprintf (stderr, "  F6\n");
	fprintf (stderr, "    Turn off all active notes.\n");
	fprintf (stderr, "  F7, F8\n");
	fprintf (stderr, "    Change base key (note played by Q and Z).\n");
	fprintf (stderr, "\n");
	fprintf (stderr, "Mouse usage:\n");
	fprintf (stderr, "  Click on the note to be played; velocity is ");
	fprintf (stderr, "selected by\n  mouse buttons.\n");
}

/* parse command line options */

void	parse_options (int argc, char **argv)
{
	int	i, j, k;

	j = argc; k = 1;
	while (--j) {
		if ((!(strcmp (argv[k], "-h")))		/* print usage */
		    || (!(strcmp (argv[k], "-help")))
		    || (!(strcmp (argv[k], "--help")))) {
			print_usage ();
			exit (0);
		/* window size */
		} else if ((!(strcmp (argv[k], "-w")))
			   || (!(strcmp (argv[k], "--winsize")))) {
			if (--j < 1) {
				fprintf (stderr, "missing argument for ");
				fprintf (stderr, "--winsize\n");
				print_usage ();
				exit (-1);
			}
			i = (int) atoi (argv[++k]);
			if ((i < 0) || (i > 3)) {
				fprintf (stderr, "invalid window size\n");
				print_usage ();
				exit (-1);
			}
			cur_size = i;
		/* MIDI format output */
		} else if ((!(strcmp (argv[k], "-M")))
			   || (!(strcmp (argv[k], "--midi")))) {
			out_format = 1;
		/* score format output */
		} else if ((!(strcmp (argv[k], "-S")))
			   || (!(strcmp (argv[k], "--score")))) {
			out_format = 0;
		/* output file */
		} else if ((!(strcmp (argv[k], "-o")))
			   || (!(strcmp (argv[k], "--output")))) {
			if (--j < 1) {
				fprintf (stderr, "missing argument for ");
				fprintf (stderr, "--output\n");
				print_usage ();
				exit (-1);
			}
			outfile_name = argv[++k];
		} else {
			fprintf (stderr, "invalid option: %s\n", argv[k]);
			print_usage ();
			exit (-1);
		}
		k++;
	}
}

