
/* draw_keys.c -- written by Istvan Varga, Apr 2002 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <gtk/gtk.h>
#include "draw_keys.h"
#include "events.h"

unsigned int	color0 = 0x00000000;	/* key border			*/
unsigned int	color1 = 0x00FFFFFF;	/* white key			*/
unsigned int	color2 = 0x00000000;	/* black key			*/
unsigned int	color3 = 0x00C0FFFF;	/* A (440 Hz) key		*/
unsigned int	color4 = 0x00D0D0D0;	/* white key / active		*/
unsigned int	color5 = 0x00707070;	/* black key / active		*/
unsigned int	color6 = 0x0090D0D0;	/* A (440 Hz) key / active	*/

int	key_sizes[4][7] =
{
	/* keyboard size 0: 450x45 pixels */
	{
		26,		/* black key height			*/
		19,		/* total height - black key height	*/
		6,		/* white key total width		*/
		3,		/* black key width			*/
		4,		/* white key width 1 (C, F)		*/
		3,		/* white key width 2 (D, G, A)		*/
		4		/* white key width 3 (E, B)		*/
	},
	/* keyboard size 1: 600x60 pixels */
	{
		35,		/* black key height			*/
		25,		/* total height - black key height	*/
		8,		/* white key total width		*/
		5,		/* black key width			*/
		5,		/* white key width 1 (C, F)		*/
		3,		/* white key width 2 (D, G, A)		*/
		5		/* white key width 3 (E, B)		*/
	},
	/* keyboard size 2: 750x75 pixels */
	{
		43,		/* black key height			*/
		32,		/* total height - black key height	*/
		10,		/* white key total width		*/
		6,		/* black key width			*/
		7,		/* white key width 1 (C, F)		*/
		4,		/* white key width 2 (D, G, A)		*/
		6		/* white key width 3 (E, B)		*/
	},
	/* keyboard size 3: 1050x105 pixels */
	{
		61,		/* black key height			*/
		44,		/* total height - black key height	*/
		14,		/* white key total width		*/
		8,		/* black key width			*/
		10,		/* white key width 1 (C, F)		*/
		6,		/* white key width 2 (D, G, A)		*/
		9		/* white key width 3 (E, B)		*/
	}
};

int		cur_size = 1;		/* current size			*/

int		img_width;		/* width in pixels		*/
int		img_height;		/* height in pixels		*/

GdkVisual	*visual;
GdkGC		*gc1;
GdkWindow	*window;

GdkWindowAttr	attr;
char            win_title[14] = "MIDI keyboard";

int		prv_active_keys[128];	/* previous key state		*/

/* calculate image width and height from current size */

void	get_size (void)
{
	img_height = key_sizes[cur_size][0] + key_sizes[cur_size][1];
	img_width = key_sizes[cur_size][2] * 75;
}

/* open window */

void	create_window (void)
{
	int	i;

	gdk_rgb_init ();
	/* calculate image size */
	get_size ();
	/* initialize window */
	visual = gdk_visual_get_best ();
	attr.title = win_title;
	attr.event_mask = GDK_ALL_EVENTS_MASK;
	attr.x = 0; attr.y = 0;
	attr.width = img_width; attr.height = img_height;
	attr.wclass = GDK_INPUT_OUTPUT;
	attr.visual = visual;
	attr.colormap = gdk_colormap_get_system ();
	attr.window_type = GDK_WINDOW_TOPLEVEL;
	attr.cursor = NULL;
	attr.wmclass_name = "keyboard";
	attr.wmclass_class = "main";
	attr.override_redirect = FALSE;
	/* create new window and make it visible */
	window = gdk_window_new (NULL, &attr, 0);
	gdk_window_show (window);
	gc1 = gdk_gc_new (window);
	gdk_gc_set_function (gc1, GDK_COPY);
	gdk_gc_set_fill (gc1, GDK_SOLID);
	gdk_rgb_gc_set_foreground (gc1, 0x00000000);
	gdk_rgb_gc_set_background (gc1, 0x00FFFFFF);
	/* disable key repeat */
	gdk_key_repeat_disable ();
	/* clear previous key state */
	for (i = 0; i < 128; i++) prv_active_keys[i] = -2;
}

/* close window */

void	close_window (void)
{
	/* enable key repeat again */
	gdk_key_repeat_restore ();
}

/* draw keyboard based on current key state */

void	draw_keyboard (int mode)
{
	int		i, j, k, l, z, s0, s1, s2, s3, s4, s5, s6;
	unsigned int	c;

	s0 = key_sizes[cur_size][0];	/* black key height		*/
	s1 = key_sizes[cur_size][1];	/* total height - b. key height	*/
	s2 = key_sizes[cur_size][2];	/* white key total width	*/
	s3 = key_sizes[cur_size][3];	/* black key width		*/
	s4 = key_sizes[cur_size][4];	/* white key width 1 (C, F)	*/
	s5 = key_sizes[cur_size][5];	/* white key width 2 (D, G, A)	*/
	s6 = key_sizes[cur_size][6];	/* white key width 3 (E, B)	*/

	/* z = 4 if entire window should be redrawn */
	z = mode & 4;
	if (z) {
		/* key borders */
		gdk_rgb_gc_set_foreground (gc1, color0);
		gdk_draw_line (window, gc1, 0, img_height - 1,
					    img_width - 1, img_height - 1);
		i = 128; l = -1;
		do {
			l += s2;
			gdk_draw_line (window, gc1, l, 0, l, img_height - 2);
		} while (--i);
	}
	j = s4;				/* black key x pos */
	l = 0;				/* white key x pos */
	i = k = 0;			/* key number (i: 0-127, k: 0-11) */
	/* draw keys */
	do {
		/* update only if state changed or redraw window was reqd */
		if ((active_keys[i] != prv_active_keys[i]) || z) {
			/* save new state */
			prv_active_keys[i] = active_keys[i];
			if (((k >= 5) && (k & 1)) || ((k < 5) && !(k & 1))) {
				/* white keys */
				if (i == 69) {	/* A (440 Hz) */
					c = (unsigned int)
					    (active_keys[i] ? color6 : color3);
				} else {
					c = (unsigned int)
					    (active_keys[i] ? color4 : color1);
				}
				/* set color */
				gdk_rgb_gc_set_foreground (gc1, c);
				if ((k == 0) || (k == 5)) {
					/* C, F */
					gdk_draw_rectangle (window, gc1, 1,
							    l, 0, s4, s0);
				} else if ((k == 4) || (k == 11)
					   || (i == 127)) {
					/* E, B */
					gdk_draw_rectangle (window, gc1, 1,
							    l + s2 - s6 - 1, 0,
							    s6, s0);
				} else {
					/* D, G, A */
					gdk_draw_rectangle (window, gc1, 1,
							    l + s2 - s6 - 1, 0,
							    s5, s0);
				}
				gdk_draw_rectangle (window, gc1, 1,
						    l, s0, s2 - 1, s1 - 1);
			} else {
				/* black keys */
				c = (unsigned int)
				    (active_keys[i] ? color5 : color2);
				/* set color */
				gdk_rgb_gc_set_foreground (gc1, c);
				gdk_draw_rectangle (window, gc1, 1,
						    j, 0, s3, s0);
			}
		}
		/* new x coordinate */
		if (((k >= 5) && (k & 1)) || ((k < 5) && !(k & 1))) {
			/* white keys */
			l += s2;
			if ((k == 4) || (k == 11)) {
				/* skip E# and B# */
				j += s2;
			}
		} else {
			/* black keys */
			j += s2;
		}
		k = (k == 11 ? 0 : k + 1);
	} while (++i < 128);
}

/* get mouse pointer coordinates, and convert to key number */
/* return -1 if pointer is not in keyboard area */

int	get_coord (void)
{
	gint		x, y, m, n, o;
	GdkModifierType	mask;

	mask = GDK_MODIFIER_MASK;
	gdk_window_get_pointer (window, &x, &y, &mask);
	/* check for valid coordinates */
	if ((x < 0) || (x >= img_width) || (y < 0) || (y >= img_height))
		return -1;
	/* calculate key number */
	n = (x / key_sizes[cur_size][2]) << 1;
	m = n % 14;
	o = 12 * (n / 14) + m;
	if (y < key_sizes[cur_size][0]) {		/* black keys */
		y = x - ((n >> 1) * key_sizes[cur_size][2]);
		switch (m) {
		case 0:			/* C */
		case 6:			/* F */
			if (y >= key_sizes[cur_size][4]) o++;
			break;
		case 4:			/* E */
		case 12:		/* B */
			if (y < (key_sizes[cur_size][2]
				 - key_sizes[cur_size][6] - 1)) o--;
			break;
		default:		/* D, G, A */
			if (y >= key_sizes[cur_size][4]) {
				o++;
			} else if (y < (key_sizes[cur_size][2]
					- key_sizes[cur_size][6] - 1)) {
				o--;
			}
		}
	}
	if (m > 4) o--;		/* correct for missing E# */
	if (o > 127) o = 127;
	return (int) o;
}

