
/* midi_out.c -- written by Istvan Varga, Apr 2002 */

#include <stdio.h>
#include <stdlib.h>
#include "midi_out.h"
#include "events.h"

FILE		*outfile = NULL;	/* output file			     */
char		*outfile_name = NULL;	/* output file name		     */

int		out_format = 1;		/* output format (0: score, 1: MIDI) */
int		off_vel = 64;		/* note-off velocity		     */
int		channel = 1;		/* MIDI channel (1 - 16)	     */
int		midi_program = -1;	/* MIDI program number (1 - 128)     */

/* clear list of active keys */

void	clear_active_keys (void)
{
	int	i;

	for (i = 0; i < 127; i++) active_keys[i] = 0;
}

/* send note-off message */

void	send_note_off (int notnum)
{
	if (!(active_keys[notnum])) return;	/* nothing to do */

	if (out_format) {				/* MIDI */
		putc (0x80 + channel - 1, outfile);
		putc (notnum, outfile);
		putc (off_vel, outfile);
	} else {					/* score */
		if (notnum > 0) {
			fprintf (outfile, "i %d 0 0.01 %d %d\n",
					  notnum, notnum, off_vel);
		}
	}
	/* remove from active list */
	active_keys[notnum] = 0;
}

/* send note-on message */

void	send_note_on (int notnum, int veloc)
{
	if (active_keys[notnum]) {	/* if already active, turn */
		send_note_off (notnum);	/* it off first		   */
	}
	if (out_format) {				/* MIDI */
		putc (0x90 + channel - 1, outfile);
		putc (notnum, outfile);
		putc ((veloc > 0 ? veloc : 1), outfile);
	} else {					/* score */
		if (notnum > 0) {
			fprintf (outfile, "i %d 0 -1 %d %d\n",
					  notnum, notnum, veloc);
		}
	}
	/* add to active list */
	active_keys[notnum] = 1;
}

/* turn off all active notes */

void	all_notes_off (void)
{
	int	i;

	for (i = 0; i < 128; i++) send_note_off (i);
}

/* change current channel */

void	set_channel (int chn)
{
	all_notes_off ();
	channel = chn;
	if (channel < 1) channel = 1;
	if (channel > 16) channel = 16;
}

/* send program change */

void	send_program_change (int pgm)
{
	all_notes_off ();
	midi_program = pgm;
	if (midi_program < 1) midi_program = 1;
	if (midi_program > 128) midi_program = 128;
	if (out_format) {
		putc (0xC0 + channel - 1, outfile);
		putc (midi_program - 1, outfile);
	}
}

